module Podjumper
  class Command
    def initialize(info)
      @namespace = info.namespace
      @pod       = info.pod_name
      @container = info.container_name
    end

    def exec(command)
      IO.popen(exec_command + command, in: :in) do |io|
        io.each_char(&method(:print))
      end
    end

    def logs
      tty = TTY::Command.new(printer: :null)
      tty.run(*logs_command) do |out, _|
        puts out unless ping?(out)
      end
    end

    private

    attr_reader :namespace, :pod, :container

    def exec_command
      [
        'kubectl',
        'exec',
        '-ti', pod,
        '-c', container,
        '--namespace', namespace,
        '--'
      ]
    end

    def logs_command
      [
        'kubectl',
        'logs',
        pod,
        container,
        '--namespace', namespace,
        '-f',
        '--since=1s'
      ]
    end

    def ping?(log_line)
      ping_matchers.any? { |regex| log_line =~ regex }
    end

    def ping_matchers
      [/api\/ping/, /Api::SystemController#ping/, /Completed 200 OK/]
    end
  end
end
