using Refinements

module Podjumper
  JumpInfo = Struct.new(:namespace, :pod, :container_name) do
    def subdomain
      pod&.subdomain
    end

    def pod_name
      pod&.name
    end
  end

  class << JumpInfo
    def load(args)
      args = args.dup
      namespace = disambiguate_namespace(args.shift)
      pod = disambiguate_pod(namespace, args.shift)
      container = disambiguate_container(pod, args.shift)

      new(namespace, pod, container)
    end

    def load_upto_pod(args)
      args = args.dup
      namespace = disambiguate_namespace(args.shift)
      pod = disambiguate_pod(namespace, args.shift)

      new(namespace, pod, nil)
    end

    def load_upto_namespace(args)
      args = args.dup
      namespace = disambiguate_namespace(args.shift)

      new(namespace, nil, nil)
    end

    def load_rails_from_subdomain(subdomain)
      pod = Pod.where(subdomain: subdomain).first
      new(pod.namespace, pod, pod.container('rails'))
    end

    private

    def disambiguate_namespace(query)
      disambiguate(namespaces, query: query, title: 'Namespaces')
    end

    def disambiguate_pod(namespace, query)
      all_pods = Pod.where(namespace: namespace)
      subdomains = all_pods.map(&:subdomain).uniq
      subdomain = disambiguate(subdomains, query: query, title: "Subdomains in namespace #{namespace.inspect}")
      pods = all_pods.select { |pod| pod.subdomain == subdomain }

      disambiguate(pods, title: "Pods in #{namespace} with subdomain #{subdomain.inspect}", &:name)
    end

    def disambiguate_container(pod, query)
      disambiguate(pod.container_names, query: query, title: "Containers in pod #{pod.name.inspect}")
    end

    def disambiguate(choices, query: nil, title: nil, &choice_name)
      choice_name ||= :itself
      matches = matching_choices(choices, query, &choice_name)
      if matches.count == 1
        matches.first
      else
        puts title if title
        cli.ask_for_choice(matches, &choice_name)
      end
    end

    def matching_choices(choices, query)
      return choices if query.nil?
      choices.select { |c| yield(c) =~ /#{query}/ }
    end

    def cli
      @cli ||= HighLine.new
    end
  end
end
