module Podjumper
  class Pod
    def initialize(description)
      @description = description
      validate_resource!
      validate_version!
    end

    def namespace
      metadata['namespace']
    end

    def subdomain
      labels['subdomain']
    end

    def name
      metadata['name']
    end

    def labels
      metadata['labels'] || {}
    end

    def container(name)
      container_names.find { |container_name| container_name =~ /#{name}/ }
    end

    def container_names
      containers.map { |container| container['name'] }
    end

    def to_h
      JSON.parse(to_json)
    end

    def to_json
      description.to_json
    end

    private

    attr_reader :description

    def kind
      description['kind']
    end

    def api_version
      description['apiVersion']
    end

    def metadata
      description['metadata']
    end

    def containers
      description['spec']['containers']
    end

    def validate_resource!
      raise "Expected a pod, got #{kind.inspect}" if kind != 'Pod'
    end

    def validate_version!
      raise "Invalid API version #{api_version.inspect}" if api_version != 'v1'
    end
  end

  class << Pod
    def where(**opts)
      output = TTY::Command.new(printer: :null).run(*_get_pods(opts)).out

      JSON.parse(output)
          .fetch('items', [])
          .map { |h| Pod.new(h) }
    end

    def _get_pods(namespace: nil, subdomain: nil)
      ns_flag = namespace ? "--namespace=#{namespace}" : '--all-namespaces'
      sd_flag = subdomain ? "--selector=subdomain=#{subdomain}" : nil
      %w[kubectl get pods --output=json]
        .push(ns_flag, sd_flag)
        .compact
    end
  end
end
